/**
  *  \file client/scr-sim.h
  *
  *  (c) 2001-2012 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_SCR_SIM_H
#define PCC_V2_CLIENT_SCR_SIM_H

#include "game/sim-if.h"
#include "ui/window.h"
#include "ui/listbox.h"

class GSimState;
class WSimList;

class WSimScreen : public UIWindow {
    ObjectHolder h;

    WSimList& ed_list;

    void init();
 public:
    WSimScreen();
    ~WSimScreen();

    void editPlanet();
    void editOptions();
    void loadFile();
    void saveFile();
    void doFleetCostComparison();
    void runSimulation();
    void writeToGame();
    void updateFromGame();
    bool handleEvent(const UIEvent& event, bool second_pass);
    void onContextMenu();
    void sortBy(bool compare(const GSimShip& a, const GSimShip& b));

    static GSimState sim_state;
    static GSimOptions sim_opts;

    static bool isExpGame();
};

/** List of battle participants. */
class WSimList : public UIListbox {
    GSimState& state;
    const bool with_planet;
    SignalConnection conn_ship_changed;
    SignalConnection conn_planet_changed;
    SignalConnection conn_structure_changed;

 protected:
    void updateCount();
    void onShipChanged(int index);
    void onPlanetChanged();
    void onStructureChanged();
 public:
    WSimList(GSimState& state, bool with_planet);
    ~WSimList();
    void drawPart(GfxCanvas& can, int from, int to);

    GSimState& getSimState()
        { return state; }
};

void initSimulatorState();
void doneSimulatorState();

int doSimulator(GSimulatorGameInterface& game_iface);

string_t getShipName(const GSimShip& sh);

extern const struct ModuleInitializer mod_sim[];


#endif
