/**
  *  \file client/sound-conf.cc
  *  \brief Sound configuration dialog
  *
  *  (c) 2001-2012 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/sound-conf.h"
#include "sound/sound.h"
#include "ui/ui.h"
#include "ui/layout.h"
#include "ui/window.h"
#include "util/nls.h"
#include "gfx/palette.h"
#include "ui/commands.h"
#include "util/confpars.h"
#include "cpluslib/holder.h"
#include "ui/widgets/focusiterator.h"
#include "ui/widgets/button.h"
#include "ui/widgets/radiobutton.h"
#include "ui/widgets/checkbox.h"
#include "ui/uiglobal.h"

/** Sound configuration dialog.
    \param config Configuration object to edit.
    \return true if user changed configuration */
bool
configureSoundDialog(SfxConfig& config)
{
    UICheckbox
        cb_sound  (100, _("Enable sound"), 'e', 2),
        cb_16     (200, _("16 Bit"), 'b', 2),
        cb_stereo (201, _("Stereo"), 's', 2),
        cb_reverse(202, _("Reverse stereo"), 'r', 2),
        cb_headphone(203, _("Headphone"), 'h', 2);

    UIRadioButton::Value rbv_freq(config.frequency >= 44000   ? 2
                                  : config.frequency >= 22000 ? 1 : 0);

    UIGroup
        gtop(UIHBoxLayout::instance5),
        gbot(UIHBoxLayout::instance5),
        gleft(UIVBoxLayout::instance0, 300),
        gright(UIVBoxLayout::instance0, 301);

    ObjectHolder holder;
    UIWindow win(_("Sound Setup"), UIWindow::BLUE_WINDOW, COLOR_GRAY);
    win.setLayout(UIVBoxLayout::instance5);

    gleft.add(cb_sound);
    gleft.add(holder.add(new UIRadioButton(101, _("11 kHz"), '1', rbv_freq, 0)));
    gleft.add(holder.add(new UIRadioButton(102, _("22 kHz"), '2', rbv_freq, 1)));
    gleft.add(holder.add(new UIRadioButton(103, _("44 kHz"), '4', rbv_freq, 2)));

    gright.add(cb_16);
    gright.add(cb_stereo);
    gright.add(cb_reverse);
    gright.add(cb_headphone);

    gtop.add(gleft);
    gtop.add(gright);
    gtop.add(holder.add(new UIFocusIterator(100, 103, UIFocusIterator::fi_Vert)));
    gtop.add(holder.add(new UIFocusIterator(200, 203, UIFocusIterator::fi_Vert)));
    gtop.add(holder.add(new UIFocusIterator(300, 301, UIFocusIterator::fi_Horiz | UIFocusIterator::fi_Tab)));

    gbot.add(holder.add(new UISpacer()));
    gbot.add(holder.add(new UIButton(cm_Enter, _("OK"), SDLK_RETURN)).addStop());
    gbot.add(holder.add(new UIButton(cm_Escape, _("Cancel"), SDLK_ESCAPE)).addStop());

    win.add(gtop);
    win.add(gbot);
    win.pack();

    cb_sound.setFocusedWidget();
    cb_sound.setValue(config.enabled);
    cb_16.setValue(config.sixteen_bits);
    cb_stereo.setValue(config.stereo);
    cb_reverse.setValue(config.reverse_stereo);
    cb_headphone.setValue(config.headphone);

    ui_root->centerWidget(win);
    ui_root->add(win);
    int rv = ui_root->run();
    ui_root->close(win);

    if (rv == cm_Enter) {
        config.enabled        = cb_sound.getValue();
        config.sixteen_bits   = cb_16.getValue();
        config.stereo         = cb_stereo.getValue();
        config.reverse_stereo = cb_reverse.getValue();
        config.headphone      = cb_headphone.getValue();
        config.frequency      = 11025 << rbv_freq.get();
        return true;
    } else {
        return false;
    }
}
