/**
  *  \file client/tiles/commenttile.cc
  *
  *  (c) 2001-2012 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/tiles/commenttile.h"
#include "ui/uiglobal.h"
#include "gfx/gfxcontext.h"
#include "game/ship.h"
#include "game/planet.h"
#include "int/intglobal.h"
#include "util/nls.h"
#include "int/data.h"

WCommentTile::WCommentTile(GObjectType& type)
    : WObjectTypeObserverTile(GfxPoint(315, font_heights[FONT_TITLE]), type),
      btn_f9(_("F9"), SDLK_F9)
{
    init();
}

WCommentTile::~WCommentTile()
{ }

void
WCommentTile::init()
{
    setLayout(RightJustified, 0, true);
    addControl(GfxRect(0, 0, 30, font_heights[FONT_TITLE]), btn_f9);
}

void
WCommentTile::drawData(GfxCanvas& can)
{
    GfxContext ctx;
    ctx.useCanvas(can);
    ctx.useColorScheme(getSkin());
    ctx.useFont(*fonts[FONT_NORMAL]);
    ctx.setSolidBackground();
    ctx.setColor(UIColor::tc_Static);
    ctx.setTextAlign(0, 1);

    string_t text;
    if (GPlanet* pl = dynamic_cast<GPlanet*>(getObject())) {
        if (IntValue* iv = getPlanetProperties(pl->getId())[pp_Comment]) {
            text = iv->toString(false);
        }
        if (text.size()) {
            ctx.setColor(UIColor::tc_Yellow);
        } else {
            text = _("(press [F9] to add a note)");
        }
    } else if (GShip* sh = dynamic_cast<GShip*>(getObject())) {
        if (IntValue* iv = getShipProperties(sh->getId())[sp_Comment]) {
            text = iv->toString(false);
        }
        if (text.size()) {
            ctx.setColor(UIColor::tc_Yellow);
        } else {
            text = _("(press [F9] to add a note)");
        }
    } else {
        // leave blank
    }

    outTextF(ctx, 5, getExtent().h/2, getExtent().w - 35, text);
}
