/**
  *  \file client/widgets/playerchoose.h
  *
  *  (c) 2001-2012 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_WIDGETS_PLAYERCHOOSE_H
#define PCC_V2_CLIENT_WIDGETS_PLAYERCHOOSE_H

#include <vector>
#include "ui/listbox.h"
#include "io/dirs.h"
#include "game/racename.h"
#include "game/playerset.h"
#include "game/struct.h"

class WPlayerChooser : public UIListbox {
 public:
    WPlayerChooser(int id = 0);
    ~WPlayerChooser();

    void addCommand(string_t name);
    void setDirectory(Ptr<Directory> dir);
    bool isAnyRaceAccessible() const;

    void drawPart(GfxCanvas& can, int from, int to);
    bool isAccessible(int index);

    void getLayoutInfo(LayoutInfo& info);

    Signal1<int> sig_command;
    Signal1<int> sig_player;

 protected:
    virtual void scan(Directory& dir) = 0;
    void setInfo(int player, bool avail, string_t text);

 private:
    void onClick(int);

    GPlayerSet available_players;
    string_t player_info[NUM_PLAYERS];
    GRaceNameList player_names;
    std::vector<string_t> commands;
};

inline bool
WPlayerChooser::isAnyRaceAccessible() const
{
    return available_players.nonempty();
}

#endif
