/**
  *  \file u/t_gfx_gfxutil.cc
  *
  *  (c) 2001-2012 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "gfx/gfxutil.h"

#include <sstream>
#include "u/t_gfx.h"

void
GfxGfxUtilTestSuite::testFillPattern()
{
    // Constructors, isBlank, isBlack
    {
        GfxFillPattern a;
        TS_ASSERT(a.isBlank());
        TS_ASSERT(!a.isBlack());
    }
    {
        GfxFillPattern a(uint8_t(0));
        TS_ASSERT(a.isBlank());
        TS_ASSERT(!a.isBlack());
    }
    {
        GfxFillPattern a(1);
        TS_ASSERT(!a.isBlank());
        TS_ASSERT(!a.isBlack());
    }
    {
        GfxFillPattern a(0xFF);
        TS_ASSERT(!a.isBlank());
        TS_ASSERT(a.isBlack());
    }
    {
        GfxFillPattern a((const uint8_t*) "\xff\xff\xff\xff\xff\xff\xff\xff");
        TS_ASSERT(a.isBlack());
        TS_ASSERT(!a.isBlank());
    }
    {
        GfxFillPattern a((const uint8_t*) "\xff\xff\xff\xff\xff\xff\xff\x00");
        TS_ASSERT(!a.isBlack());
        TS_ASSERT(!a.isBlank());
    }

    // Operators
    {
        GfxFillPattern a((const uint8_t*) "\x01\0\0\0\0\0\x30\0");
        TS_ASSERT(!a.isBlank());
        TS_ASSERT(!a.isBlack());
        TS_ASSERT_EQUALS(a[0], 1);
        TS_ASSERT_EQUALS(a[1], 0);
        TS_ASSERT_EQUALS(a[2], 0);
        TS_ASSERT_EQUALS(a[3], 0);
        TS_ASSERT_EQUALS(a[4], 0);
        TS_ASSERT_EQUALS(a[5], 0);
        TS_ASSERT_EQUALS(a[6], 0x30);
        TS_ASSERT_EQUALS(a[7], 0);

        a.shiftLeft(2);
        TS_ASSERT_EQUALS(a[0], 4);
        TS_ASSERT_EQUALS(a[1], 0);
        TS_ASSERT_EQUALS(a[2], 0);
        TS_ASSERT_EQUALS(a[3], 0);
        TS_ASSERT_EQUALS(a[4], 0);
        TS_ASSERT_EQUALS(a[5], 0);
        TS_ASSERT_EQUALS(a[6], 0xC0);
        TS_ASSERT_EQUALS(a[7], 0);

        a.shiftUp(3);
        TS_ASSERT_EQUALS(a[0], 0);
        TS_ASSERT_EQUALS(a[1], 0);
        TS_ASSERT_EQUALS(a[2], 0);
        TS_ASSERT_EQUALS(a[3], 0xC0);
        TS_ASSERT_EQUALS(a[4], 0);
        TS_ASSERT_EQUALS(a[5], 4);
        TS_ASSERT_EQUALS(a[6], 0);
        TS_ASSERT_EQUALS(a[7], 0);

        a.shiftRight(5);
        TS_ASSERT_EQUALS(a[0], 0);
        TS_ASSERT_EQUALS(a[1], 0);
        TS_ASSERT_EQUALS(a[2], 0);
        TS_ASSERT_EQUALS(a[3], 6);
        TS_ASSERT_EQUALS(a[4], 0);
        TS_ASSERT_EQUALS(a[5], 0x20);
        TS_ASSERT_EQUALS(a[6], 0);
        TS_ASSERT_EQUALS(a[7], 0);

        a.shiftDown(1);
        TS_ASSERT_EQUALS(a[0], 0);
        TS_ASSERT_EQUALS(a[1], 0);
        TS_ASSERT_EQUALS(a[2], 0);
        TS_ASSERT_EQUALS(a[3], 0);
        TS_ASSERT_EQUALS(a[4], 6);
        TS_ASSERT_EQUALS(a[5], 0);
        TS_ASSERT_EQUALS(a[6], 0x20);
        TS_ASSERT_EQUALS(a[7], 0);

        a.flipVertical();
        TS_ASSERT_EQUALS(a[0], 0);
        TS_ASSERT_EQUALS(a[1], 0x20);
        TS_ASSERT_EQUALS(a[2], 0);
        TS_ASSERT_EQUALS(a[3], 6);
        TS_ASSERT_EQUALS(a[4], 0);
        TS_ASSERT_EQUALS(a[5], 0);
        TS_ASSERT_EQUALS(a[6], 0);
        TS_ASSERT_EQUALS(a[7], 0);

        a.flipHorizontal();
        TS_ASSERT_EQUALS(a[0], 0);
        TS_ASSERT_EQUALS(a[1], 4);
        TS_ASSERT_EQUALS(a[2], 0);
        TS_ASSERT_EQUALS(a[3], 0x60);
        TS_ASSERT_EQUALS(a[4], 0);
        TS_ASSERT_EQUALS(a[5], 0);
        TS_ASSERT_EQUALS(a[6], 0);
        TS_ASSERT_EQUALS(a[7], 0);

        a.invert();
        TS_ASSERT_EQUALS(a[0], 0xFF);
        TS_ASSERT_EQUALS(a[1], 0xFB);
        TS_ASSERT_EQUALS(a[2], 0xFF);
        TS_ASSERT_EQUALS(a[3], 0x9F);
        TS_ASSERT_EQUALS(a[4], 0xFF);
        TS_ASSERT_EQUALS(a[5], 0xFF);
        TS_ASSERT_EQUALS(a[6], 0xFF);
        TS_ASSERT_EQUALS(a[7], 0xFF);

        a ^= 0xF0;
        TS_ASSERT_EQUALS(a[0], 0x0F);
        TS_ASSERT_EQUALS(a[1], 0x0B);
        TS_ASSERT_EQUALS(a[2], 0x0F);
        TS_ASSERT_EQUALS(a[3], 0x6F);
        TS_ASSERT_EQUALS(a[4], 0x0F);
        TS_ASSERT_EQUALS(a[5], 0x0F);
        TS_ASSERT_EQUALS(a[6], 0x0F);
        TS_ASSERT_EQUALS(a[7], 0x0F);

        a ^= GfxFillPattern((const uint8_t*) "\x08\x1B\x18\x0B\x08\x1B\x18\x0B");
        TS_ASSERT_EQUALS(a[0], 0x07);
        TS_ASSERT_EQUALS(a[1], 0x10);
        TS_ASSERT_EQUALS(a[2], 0x17);
        TS_ASSERT_EQUALS(a[3], 0x64);
        TS_ASSERT_EQUALS(a[4], 0x07);
        TS_ASSERT_EQUALS(a[5], 0x14);
        TS_ASSERT_EQUALS(a[6], 0x17);
        TS_ASSERT_EQUALS(a[7], 0x04);

        a |= 0x40;
        TS_ASSERT_EQUALS(a[0], 0x47);
        TS_ASSERT_EQUALS(a[1], 0x50);
        TS_ASSERT_EQUALS(a[2], 0x57);
        TS_ASSERT_EQUALS(a[3], 0x64);
        TS_ASSERT_EQUALS(a[4], 0x47);
        TS_ASSERT_EQUALS(a[5], 0x54);
        TS_ASSERT_EQUALS(a[6], 0x57);
        TS_ASSERT_EQUALS(a[7], 0x44);

        a |= GfxFillPattern((const uint8_t*) "\x08\x02\x02\x08\x08\x02\x02\x08");
        TS_ASSERT_EQUALS(a[0], 0x4F);
        TS_ASSERT_EQUALS(a[1], 0x52);
        TS_ASSERT_EQUALS(a[2], 0x57);
        TS_ASSERT_EQUALS(a[3], 0x6C);
        TS_ASSERT_EQUALS(a[4], 0x4F);
        TS_ASSERT_EQUALS(a[5], 0x56);
        TS_ASSERT_EQUALS(a[6], 0x57);
        TS_ASSERT_EQUALS(a[7], 0x4C);

        a &= 0x11;
        TS_ASSERT_EQUALS(a[0], 0x01);
        TS_ASSERT_EQUALS(a[1], 0x10);
        TS_ASSERT_EQUALS(a[2], 0x11);
        TS_ASSERT_EQUALS(a[3], 0x00);
        TS_ASSERT_EQUALS(a[4], 0x01);
        TS_ASSERT_EQUALS(a[5], 0x10);
        TS_ASSERT_EQUALS(a[6], 0x11);
        TS_ASSERT_EQUALS(a[7], 0x00);

        a &= GfxFillPattern((const uint8_t*) "\xFF\xF0\x0F\xFF\xF0\x0F\xFF\xF0");
        TS_ASSERT_EQUALS(a[0], 0x01);
        TS_ASSERT_EQUALS(a[1], 0x10);
        TS_ASSERT_EQUALS(a[2], 0x01);
        TS_ASSERT_EQUALS(a[3], 0x00);
        TS_ASSERT_EQUALS(a[4], 0x00);
        TS_ASSERT_EQUALS(a[5], 0x00);
        TS_ASSERT_EQUALS(a[6], 0x11);
        TS_ASSERT_EQUALS(a[7], 0x00);
    }

    // Predefined
    TS_ASSERT(SOLID_FILL.isBlack());
    TS_ASSERT(!SOLID_FILL.isBlank());

    TS_ASSERT(!GRAY50_FILL.isBlack());
    TS_ASSERT(!GRAY50_FILL.isBlank());

    TS_ASSERT(!GRAY25_FILL.isBlack());
    TS_ASSERT(!GRAY25_FILL.isBlank());

    TS_ASSERT(!GRAY50_FILL2.isBlack());
    TS_ASSERT(!GRAY50_FILL2.isBlank());

    TS_ASSERT(!LTSLASH_FILL.isBlack());
    TS_ASSERT(!LTSLASH_FILL.isBlank());
}

void
GfxGfxUtilTestSuite::testPoint()
{
    const GfxPoint a(1,2);
    const GfxPoint b(3,4);

    // Construction, equality, inequality
    TS_ASSERT_EQUALS(a.x, 1);
    TS_ASSERT_EQUALS(a.y, 2);
    TS_ASSERT_EQUALS(b.x, 3);
    TS_ASSERT_EQUALS(b.y, 4);
    TS_ASSERT(a == a);
    TS_ASSERT(a == GfxPoint(1,2));
    TS_ASSERT(a != b);

    // movedBy
    TS_ASSERT(a.movedBy(2, 2) == b);
    TS_ASSERT(b.movedBy(-2, -2) == a);
    TS_ASSERT(a.movedBy(GfxPoint(2, 2)) == b);
    TS_ASSERT(b.movedBy(GfxPoint(-2, -2)) == a);

    // scaledBy
    TS_ASSERT(a.scaledBy(5,6) == GfxPoint(5, 12));
    TS_ASSERT(b.scaledBy(7,8) == GfxPoint(21, 32));

    // operator<<
    std::stringstream ss;
    ss << a;
    TS_ASSERT_EQUALS(ss.str(), "(1, 2)");
}

void
GfxGfxUtilTestSuite::testRect()
{
    // Constructors
    GfxRect nullr;
    GfxRect a(10, 20, 30, 40);
    GfxRect b(10, 20, GfxPoint(30, 40));
    GfxRect c(GfxPoint(10, 20), 30, 40);
    GfxRect d(GfxPoint(10, 20), GfxPoint(30, 40));

    // Query
    TS_ASSERT_EQUALS(a.getBottomY(), 60);
    TS_ASSERT_EQUALS(a.getRightX(), 40);
    TS_ASSERT(a.getBottomRight() == GfxPoint(40, 60));
    TS_ASSERT(a.getTopLeft() == GfxPoint(10, 20));
    TS_ASSERT(a.getCenter() == GfxPoint(25, 40));
    TS_ASSERT(a.getSize() == GfxPoint(30, 40));

    // Equality
    TS_ASSERT(a == a);
    TS_ASSERT(a == b);
    TS_ASSERT(a == c);
    TS_ASSERT(a == d);
    TS_ASSERT(a != nullr);
    TS_ASSERT(a != GfxRect(10, 20, 30, 0));
    TS_ASSERT(a != GfxRect(10, 20, 0, 40));
    TS_ASSERT(a != GfxRect(10, 0, 30, 40));
    TS_ASSERT(a != GfxRect(0, 20, 30, 40));

    // exists
    TS_ASSERT(!nullr.exists());
    TS_ASSERT(a.exists());

    // contains
    TS_ASSERT(!nullr.contains(1,1));
    TS_ASSERT(!nullr.contains(0,0));
    TS_ASSERT(!a.contains(1,1));
    TS_ASSERT(!a.contains(0,0));
    TS_ASSERT(!a.contains(10,19));
    TS_ASSERT(a.contains(10,20));
    TS_ASSERT(!a.contains(40,20));

    TS_ASSERT(a.contains(GfxPoint(10,20)));
    TS_ASSERT(!a.contains(GfxPoint(40,20)));


    TS_ASSERT(a.contains(nullr));
    TS_ASSERT(a.contains(b));
    TS_ASSERT(!nullr.contains(a));
    TS_ASSERT(a.contains(GfxRect(10, 20, 10, 10)));
    TS_ASSERT(!a.contains(GfxRect(10, 20, 30, 41)));

    // clipSize
    // clipSize is the same as clipRect(GfxRect(0, 0, w, h))
    {
        GfxRect z(a);
        z.clipSize(15, 35);
        TS_ASSERT(z == GfxRect(10, 20, 5, 15));

        GfxRect y(a);
        y.clipSize(95, 35);
        TS_ASSERT(y == GfxRect(10, 20, 30, 15));

        GfxRect x(a);
        x.clipSize(15, 95);
        TS_ASSERT(x == GfxRect(10, 20, 5, 40));
    }


    // GfxPoint moveTo(GfxPoint where) throw();
    // GfxPoint moveTo(int px, int py) throw();
    // bool isIntersecting(GfxRect r) const;
    // void centerWithin(const GfxRect& other) throw();
    // void clipRect(const SDL_Rect& r) throw();
    // void grow(int dx, int dy);
    // void growSize(GfxPoint pt);
    // void growSize(int dx, int dy);
    // void include(GfxPoint pt) throw();
    // void include(const SDL_Rect& r) throw();
    // void intersect(const SDL_Rect& r) throw();
    // void moveBy(GfxPoint dist);
    // void moveBy(int dx, int dy);
    // void moveByRev(GfxPoint dist);
    // void moveByRev(int dx, int dy);
    // void moveInto(const GfxRect& other) throw();
    // void moveToEdge(const GfxRect& other, int x_pos, int y_pos, int offset) throw();

    // std::ostream& operator<<(std::ostream&, const GfxRect&);
}

void
GfxGfxUtilTestSuite::testRectSet()
{
    // GfxRectSet();
    // GfxRectSet(const GfxRect& r);

    // void intersect(const GfxRect& r);
    // void add(const GfxRect& r);
    // void remove(const GfxRect& r);
    // void clear();
    // bool contains(int x, int y) const;
    // bool contains(const GfxPoint& r) const;
    // bool contains(const GfxRect& r) const;
    // GfxRect getBoundingRectangle() const;
    // iterator begin() const;
    // iterator end() const;
    // bool empty() const;
    // std::list<GfxRect>::size_type getNumRectangles() const;
}
